/*
   Regexp Search
   Copyright (C) 1999 Jarek Woloszyn (yossa@pld.org.pl)
   $Id: regexp-search.c,v 1.1 2001/02/05 15:51:06 malekith Exp $
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
 */

#include<stdlib.h>
#include<string.h>
#include<stdio.h>
#include<ctype.h>
#include <regex.h>

/* adapted from midnight commander view regexp search */

int easy_patterns = 1;

static char *maybe_start_group(char *d, int do_group, int *was_wildcard)
{
    if (!do_group)
	return d;
    if (*was_wildcard)
	return d;
    *was_wildcard = 1;
    *d++ = '\\';
    *d++ = '(';
    return d;
}

static char *maybe_end_group(char *d, int do_group, int *was_wildcard)
{
    if (!do_group)
	return d;
    if (!*was_wildcard)
	return d;
    *was_wildcard = 0;
    *d++ = '\\';
    *d++ = ')';
    return d;
}

/* If shell patterns are on converts a shell pattern to a regular
   expression. Called by regexp_match and mask_rename. */
/* Shouldn't we support [a-fw] type wildcards as well ?? */
char *convert_pattern(char *pattern, int do_group)
{
    char *s, *d;
    char *new_pattern;
    int was_wildcard = 0;

    if (easy_patterns) {
	new_pattern = malloc(sizeof(char) * 4096);
	d = new_pattern;
	*d++ = '^';
	for (s = pattern; *s; s++, d++) {
	    switch (*s) {
	    case '*':
		d = maybe_start_group(d, do_group, &was_wildcard);
		*d++ = '.';
		*d = '*';
		break;

	    case '?':
		d = maybe_start_group(d, do_group, &was_wildcard);
		*d = '.';
		break;

	    case '.':
		d = maybe_end_group(d, do_group, &was_wildcard);
		*d++ = '\\';
		*d = '.';
		break;

	    default:
		d = maybe_end_group(d, do_group, &was_wildcard);
		*d = *s;
		break;
	    }
	}
	d = maybe_end_group(d, do_group, &was_wildcard);
	*d++ = '$';
	*d = 0;
	return new_pattern;
    } else
	return strdup(pattern);
}

int regexp_search(char *pattern, char *string)
{
    static regex_t r;
    static char *old_pattern = NULL;
    static char *old_convpattern = NULL;
    static int err = 0;
    char *convpattern;

    int i, rval;

    if (!old_pattern || strcmp(old_pattern, pattern)) {
	err = 0;
	if (old_pattern) {
	    regfree(&r);
	    free(old_pattern);
	    old_pattern = NULL;
	}
	if (old_convpattern) {
	    free(old_convpattern);
	    old_convpattern = NULL;
	}
	old_pattern = strdup(pattern);
	convpattern = convert_pattern(pattern, 0);
	if ((i = regcomp(&r, convpattern, REG_EXTENDED | REG_NOSUB))) {
	    err = i;
	    free(convpattern);
	    return i;
	}
	old_convpattern = convpattern;
    } else if (err)
	return err;
    rval = regexec(&r, string, 0, NULL, 0);
    return rval;
}
