/*
 * pi - P user Interface
 * 
 * $Id: msgbox.c,v 1.1 2001/02/05 15:51:07 malekith Exp $
 * Author: Michal Moskal <malekith@pld.org.pl>
 * include COPYING-GNU
 */

#include <pi/pi.h>
#include <string.h>
#include <stdlib.h>

struct msgbox_ctx {
	int w, h;
	int scrollable;
	char *button1, *button2;
	struct pi_wordbreak_res *wb;
	struct pi_object *form;
};

static void setup_buttons(struct msgbox_ctx *self, struct pi_object *ctx)
{
	self->button1 = pi_ptr(button_yes);
	self->button2 = pi_ptr(button_no);

	if (self->button1 == 0 || self->button2 == 0) {
		self->button2 = self->button1;
		self->button1 = pi_ptr(button_ok);
		if (self->button1 == 0 && self->button2)
			self->button1 = self->button2;
		self->button2 = 0;
		if (self->button1 == 0)
			self->button1 = "_Ok";
	}
	
	self->w = pi_strlen_alt(self->button1) + 4 + 
		(self->button2 ?
			pi_strlen_alt(self->button2) + 5 : 
			0);
}

static void setup_sizes(struct msgbox_ctx *self, struct pi_object *ctx)
{
	self->wb = pi_wordbreak(pi_ptrd(caption, ""), pi_screen_width - 30);
		
	if (self->wb->width < self->w) {
		free(self->wb);
		self->wb = pi_wordbreak(pi_ptrd(caption, ""), -self->w);
	}

	self->w = self->wb->width + 5;

	if (self->wb->nlines > pi_screen_height - 10) {
		self->w++;
		self->scrollable = 1;
		self->h = pi_screen_height - 10;
	} else {
		self->h = self->wb->nlines + 4;
	}
}

static void draw_line(struct pi_object *ctx, int line_no, int w, int hl)
{
	struct msgbox_ctx *self;

	(void)hl;

	self = pi_ptr(userdata1);
	
	pi_addstrn_fill(line_no == -1 ? 
		"~" : 
		self->wb->lines[line_no], w);
}

static void make_object(struct msgbox_ctx *self, struct pi_object *ctx)
{
	struct pi_object *s;
	int x;
	
	self->form = pi_new_window(pi_add_object(0,
		pi_tag_w, self->w,
		pi_tag_h, self->h,
		pi_tag_caption, pi_ptrd(title, ""),
	0), 0);

	s = pi_new_scrollable(pi_add_object(0,
		pi_tag_x, 2,
		pi_tag_y, 1,
		pi_tag_w, self->w - 4,
		pi_tag_h, self->h - 4,
	0), 0);
	pi_set_tag(s, pi_tag_draw_line, draw_line);
	pi_set_tag(s, pi_tag_max, self->wb->nlines);
	pi_set_tag(s, pi_tag_userdata1, self);
	if (!self->scrollable)
		s->flags &= ~pi_f_can_get_focus;
	pi_add_child(self->form, s);

	x = self->w - 2 - pi_strlen_alt(self->button1) - 4;
	
	if (self->button2) 
		x -= pi_strlen_alt(self->button2) + 5;

	pi_add(self->form, button, (0,
		pi_tag_x, x,
		pi_tag_y, self->h - 2,
		pi_tag_caption, self->button1,
		pi_tag_cc, 1,
	0));

	if (self->button2) {
		x += pi_strlen_alt(self->button1) + 5;
		pi_add(self->form, button, (0,
			pi_tag_x, x,
			pi_tag_y, self->h - 2,
			pi_tag_caption, self->button2,
			pi_tag_cc, 2,
		0));
	}
}

int pi_msgbox(struct pi_object *ctx)
{
	struct msgbox_ctx xself, *self;
	int r;
	
	memset(self = &xself, 0, sizeof(xself));

	setup_buttons(self, ctx);
	setup_sizes(self, ctx);
	make_object(self, ctx);

	pi_center(self->form);
	
	r = pi_exec(self->form);

	pi_kill(self->form);
	free(self->wb);

	return r == 1;
}
