#include <pi/pi.h>

#include "rpmpldi.h"

enum {
	cc_select = 31337,
	cc_add_deps,
	cc_remove_deps,
	cc_run,
};

struct selector_ctx {
	TPkgSet *pkgset;
	struct pi_object *form, *menu;

	int colors[3], hl_colors[3];
};

static void draw_line(struct pi_object *ctx, int line_no, int w, int hl)
{
	struct selector_ctx *self;
	char buf[100];
	TPkgInfo *p;

	if (line_no == -1) {
		pi_addstrn_fill("~", w);
		return;
	}
		
	self = pi_ptr(userdata1);
	p = pkgs_array_nth(self->pkgset, line_no);
	pi_color((hl ? self->hl_colors : self->colors)[(unsigned)p->selected]);
	snprintf(buf, sizeof(buf), "%c%-27.27s%c",
		p->selected == RPMMEN_FROM_FILE ? '<' :
		p->selected == RPMMEN_FROM_DEPS ? '*' : ' ',
		p->name,
		p->selected == RPMMEN_FROM_FILE ? '>' : ' ');
		
	pi_addstrn_fill(buf, w);
}

static void win_cc(struct pi_object *ctx, int cc)
{
	TPkgInfo *p;
	struct selector_ctx *self;
	
	self = pi_ptr(userdata1);

	switch (cc) {
	case cc_select:
		p = pkgs_array_nth(self->pkgset, pi_oint(self->menu, pos));
		p->selected = !p->selected;
		pi_set_tag(self->menu, pi_tag_pos, pi_oint(self->menu, pos)+1);
		pi_draw(self->menu);
		break;
		
	case cc_remove_deps:
		rpmpldi_remove_deps();
		pi_draw(self->menu);
		break;
		
	case cc_add_deps:
		rpmpldi_remove_deps();
		rpmpldi_add_deps();
		pi_draw(self->menu);
		break;
		
	case cc_run:
		pi_set_tag(self->form, pi_tag_modal_result, 1);
		break;
	}
}

static void make_form(struct selector_ctx *self)
{
	int wh, ww;

	wh = pi_screen_height - 5;
	ww = pi_screen_width - 20;
	
	self->form = pi_new_window(pi_add_object(0,
		pi_tag_w, ww,
		pi_tag_h, wh,
	0), 0);
	pi_center(self->form);
	pi_set_tag(self->form, pi_tag_cc_handler, win_cc);
	pi_add_child(pi_root_object, self->form);
	self->menu = pi_add(self->form, scrollable, (0,
		pi_tag_x, 1,
		pi_tag_y, 1,
		pi_tag_w, 30,
		pi_tag_h, wh - 4,
		pi_tag_cc, cc_select,
		pi_tag_flags, pi_f_can_get_focus,
		0));
	pi_set_tag(self->menu, pi_tag_draw_line, draw_line);
	pi_set_tag(self->menu, pi_tag_menu_like, 1);
	pi_set_tag(self->menu, pi_tag_max, pkgs_array_num(self->pkgset));
	pi_set_tag(self->menu, pi_tag_userdata1, self);
	pi_set_tag(self->form, pi_tag_userdata1, self);
	
	pi_add(self->form, button, (0,
		pi_tag_x, 1,
		pi_tag_y, wh - 2,
		pi_tag_cc, cc_add_deps,
		pi_tag_caption, _("_Add deps"),
	0));
	pi_add(self->form, button, (0,
		pi_tag_x, 20,
		pi_tag_y, wh - 2,
		pi_tag_cc, cc_remove_deps,
		pi_tag_caption, _("_Del deps"),
	0));
	pi_add(self->form, button, (0,
		pi_tag_x, 30,
		pi_tag_y, wh - 2,
		pi_tag_cc, cc_run,
		pi_tag_caption, _("_Run"),
	0));
}

static void set_colors(struct selector_ctx *self)
{
	self->colors[0] = pi_register_color(pi_color_lightgray, pi_color_blue);
	self->colors[1] = pi_register_color(pi_color_yellow, pi_color_blue);
	self->colors[2] = pi_register_color(pi_color_lightcyan, pi_color_blue);
	self->hl_colors[0] = pi_register_color(pi_color_black, pi_color_cyan);
	self->hl_colors[1] = pi_register_color(pi_color_yellow, pi_color_cyan);
	self->hl_colors[2] = pi_register_color(pi_color_lightcyan, pi_color_cyan);
}

void pi_selector()
{
	struct selector_ctx self;
	
	memset(&self, 0, sizeof(self));
	self.pkgset = rpmpldi_getPkgsSet();

	set_colors(&self);
	make_form(&self);

	pi_exec(self.form);
	pi_kill(self.form);
}

