//#include <pgsql/postgres.h>
#include <postgresql/postgres.h>
#include <rpm/rpmlib.h>

/* Author: Pawel Kolodziej <pawelk@pld.org.pl> */

/* Funkcja dokladana do postgres'a (plik addfunction.sql) sluzaca
 * do porownywania wersji pakietow zapisanych jako tekst
 * ([Epoch:]Version-Revision).
 * Kompilacja/linkowanie:
 *  gcc evrcmp.c --share -o /tmp/evrcmp.so /usr/lib/librpm.a /usr/lib/librpmio.a -lpopt -ldb3 -ldb1 -lz -lbz2
 */  

int2 evrcmp(text* pv1,text* pv2)
{
	char *ver1,*ver2;
	char *v1,*v2,*r1,*r2;
	int e1,e2;
	int2 ret=0;

	ver1=palloc(VARSIZE(pv1)-VARHDRSZ+1); 
	ver2=palloc(VARSIZE(pv2)-VARHDRSZ+1);
	memcpy(ver1,VARDATA(pv1),VARSIZE(pv1)-VARHDRSZ+1);
	memcpy(ver2,VARDATA(pv2),VARSIZE(pv2)-VARHDRSZ+1);
	ver1[VARSIZE(pv1)-VARHDRSZ]='\0';
	ver2[VARSIZE(pv2)-VARHDRSZ]='\0';

	v1=strchr(ver1,':');
	if(v1){
		*v1='\0';
		v1++;
		e1=atoi(ver1);
	}else{
		e1=0;
		v1=ver1;
	}
	
	v2=strchr(ver2,':');	
	if(v2){
		*v2='\0';
		v2++;
		e2=atoi(ver2);
	}else{
		e2=0;
		v2=ver2;
	}
	r1=strrchr(v1,'-');
	if(r1){
		*r1='\0';
		r1++;
	}else{
		r1="";
	}
	
	r2=strrchr(v2,'-');
	if(r2){
		*r2='\0';
		r2++;
	}else{
		r2="";
	}

	if(e1 != e2){
		ret=e1-e2;
		goto end;
	}
	ret= rpmvercmp(v1,v2);
	if(ret)
		goto end;
	ret=rpmvercmp(r1,r2);

end:
	pfree(ver1);
	pfree(ver2);
	if(ret<0)
		return -1;
	if(ret>0)
		return 1;
	return 0;
}

int2 evcmp(text* pv1,text* pv2)
{	
	char *ver1,*ver2;
	char *v1,*v2,*r1,*r2;
	int e1,e2;
	int2 ret=0;

	ver1=palloc(VARSIZE(pv1)-VARHDRSZ+1); 
	ver2=palloc(VARSIZE(pv2)-VARHDRSZ+1);
	memcpy(ver1,VARDATA(pv1),VARSIZE(pv1)-VARHDRSZ+1);
	memcpy(ver2,VARDATA(pv2),VARSIZE(pv2)-VARHDRSZ+1);
	ver1[VARSIZE(pv1)-VARHDRSZ]='\0';
	ver2[VARSIZE(pv2)-VARHDRSZ]='\0';

	v1=strchr(ver1,':');
	if(v1){
		*v1='\0';
		v1++;
		e1=atoi(ver1);
	}else{
		e1=0;
		v1=ver1;
	}
	
	v2=strchr(ver2,':');	
	if(v2){
		*v2='\0';
		v2++;
		e2=atoi(ver2);
	}else{
		e2=0;
		v2=ver2;
	}
	r1=strrchr(v1,'-');
	if(r1){
		*r1='\0';
		r1++;
	}else{
		r1="";
	}
	
	r2=strrchr(v2,'-');
	if(r2){
		*r2='\0';
		r2++;
	}else{
		r2="";
	}

	if(e1 != e2){
		ret=e1-e2;
		goto end;
	}
	ret= rpmvercmp(v1,v2);

end:
	pfree(ver1);
	pfree(ver2);
	if(ret<0)
		return -1;
	if(ret>0)
		return 1;
	return 0;
	
}
